@@ -9,7 +9,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Dec-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #----------------------------------------------------------------------
 
@@ -87,16 +87,16 @@ wxICON = 'packaging/docset/mondrian.png'
 
 # Some tools will be downloaded for the builds. These are the versions and
 # MD5s of the tool binaries currently in use.
-sipCurrentVersion = '4.19.16'
+sipCurrentVersion = '4.19.24'
 sipMD5 = {
-    'darwin'   : 'ddb27cd807cc5474ce4d557f18b709dc',
-    'win32'    : 'e381c10b799a2bd130fceb330a149626',
-    'linux32'  : '6f5438ecaf38eef35380ff6652747f8b',
-    'linux64'  : 'e8566e05d2122344ec7ce0e289877f74',
+    'darwin'   : '2a22cb7a35eb14384b0829593a366c29',
+    'win32'    : '49e0aa36397d7629fea95418452961fb',
+    'linux32'  : 'ea773f6fd92d5f23530730428a86df2f',
+    'linux64'  : 'b44a45191f5f84db10e2ba1c4cecd8ff',
 }
 
-wafCurrentVersion = '2.0.8'
-wafMD5 = 'bc393f29337cd00aef25d54280ca22e1'
+wafCurrentVersion = '2.0.19'
+wafMD5 = 'ac362b60111a59ab2df63513018d5ad8'
 
 doxygenCurrentVersion = '1.8.8'
 doxygenMD5 = {
@@ -109,6 +109,10 @@ doxygenMD5 = {
 toolsURL = 'https://wxpython.org/Phoenix/tools'
 
 
+# MS Edge code and DLLs needed for the wxWEBVIEW_BACKEND_EDGE backend
+MS_edge_version = '1.0.622.22'
+MS_edge_url = 'https://www.nuget.org/api/v2/package/Microsoft.Web.WebView2/{}'.format(MS_edge_version)
+
 #---------------------------------------------------------------------------
 
 def usage():
@@ -129,8 +133,6 @@ Usage: ./build.py [command(s)] [options]
       sip           Run sip to generate the C++ wrapper source
 
       wxlib         Build the Sphinx input files for wx.lib
-      wxpy          Build the Sphinx input files for wx.py
-      wxtools       Build the Sphinx input files for wx.tools
       sphinx        Run the documentation building process using Sphinx
 
       docset        Build Dash or Zeal compatible docsets
@@ -169,7 +171,7 @@ def main(args):
     setPythonVersion(args)
     setDevModeOptions(args)
 
-    os.environ['PYTHONPATH'] = phoenixDir()
+    os.environ['PYTHONPATH'] = os.environ.get('PYTHONPATH', '') + os.pathsep + phoenixDir()
     os.environ['PYTHONUNBUFFERED'] = 'yes'
     os.environ['WXWIN'] = wxDir()
 
@@ -448,10 +450,11 @@ def makeOptionParser():
         ("jom",            (False, "Use jom instead of nmake for the wxMSW build")),
         ("pytest_timeout", ("0",   "Timeout, in seconds, for stopping stuck test cases. (Currently not working as expected, so disabled by default.)")),
         ("pytest_jobs",    ("",    "Number of parallel processes py.test should run")),
-        ("vagrant_vms",    ("all", "Comma separated list of VM names to use for the build_vagrant command. Defaults to \"all\"")),
+        ("docker_img",     ("all", "Comma separated list of image tags to use for the build_docker command. Defaults to \"all\"")),
         ("dump_waf_log",   (False, "If the waf build tool fails then using this option will cause waf's configure log to be printed")),
         ("regenerate_sysconfig", (False, "Waf uses Python's sysconfig and related tools to configure the build. In some cases that info can be incorrect, so this option regenerates it. Must have write access to Python's lib folder.")),
         ("no_allmo",       (False, "Skip regenerating the wxWidgets message catalogs")),
+        ("no_msedge",      (False, "Do not include the MS Edge backend for wx.html2.WebView. (Windows only)")),
         ]
 
     parser = optparse.OptionParser("build options:")
@@ -578,7 +581,8 @@ def getTool(cmdName, version, MD5, envVar, platformBinary, linuxBits=False):
             # now check the MD5 if not in dev mode and it's set to None
             if not (devMode and md5 is None):
                 m = hashlib.md5()
-                m.update(open(cmd, 'rb').read())
+                with open(cmd, 'rb') as fid:
+                    m.update(fid.read())
                 if m.hexdigest() != md5:
                     _error_msg('MD5 mismatch, got "%s"\n       '
                                'expected          "%s"' % (m.hexdigest(), md5))
@@ -596,7 +600,7 @@ def getTool(cmdName, version, MD5, envVar, platformBinary, linuxBits=False):
 
             try:
                 p = subprocess.Popen([cmd, '--help'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
-                p.wait()
+                p.communicate()
             except OSError as e:
                 _error_msg('Could not execute %s, got "%s"' % (cmd, e))
                 sys.exit(1)
@@ -659,6 +663,35 @@ def getDoxCmd():
     return _doxCmd
 
 
+def getMSWebView2():
+    fname = '{}.zip'.format(MS_edge_version)
+    dest = opj(wxDir(), '3rdparty', 'webview2')
+    if not os.path.exists(dest) or not os.path.exists(opj(dest, fname)):
+        if os.path.exists(dest):
+            shutil.rmtree(dest)
+        os.makedirs(dest)
+
+        msg('Downloading microsoft.web.webview2 {}...'.format(MS_edge_version))
+        try:
+            import requests
+            resp = requests.get(MS_edge_url)
+            resp.raise_for_status()
+            msg('Connection successful...')
+            data = resp.content
+            msg('Data downloaded...')
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            sys.exit(1)
+
+        # Write the downloaded data to a local file
+        with open(opj(dest, fname), 'wb') as f:
+            f.write(data)
+
+        # Unzip it
+        from zipfile import ZipFile
+        with ZipFile(opj(dest, fname)) as zip:
+            zip.extractall(dest)
 
 
 class CommandTimer(object):
@@ -683,6 +716,7 @@ def uploadPackage(fileName, options, mask=defaultMask, keep=75):
     space is not overly consumed. It is assumed that if the files are in
     sorted order then the end of the list will be the newest files.
     """
+    fileName = os.path.relpath(fileName)
     fileName = fileName.replace('\\', '/')
     msg("Uploading %s..." % fileName)
 
@@ -763,7 +797,7 @@ def checkCompiler(quiet=False):
         # Make sure that the compiler that Python wants to use can be found.
         # It will terminate if the compiler is not found or other exceptions
         # are raised.
-        cmd = "import distutils.msvc9compiler as msvc; " \
+        cmd = "import setuptools, distutils.msvc9compiler as msvc; " \
               "mc = msvc.MSVCCompiler(); " \
               "mc.initialize(); " \
               "print(mc.cc)"
@@ -774,7 +808,7 @@ def checkCompiler(quiet=False):
         # Now get the environment variables which that compiler needs from
         # its vcvarsall.bat command and load them into this process's
         # environment.
-        cmd = "import distutils.msvc9compiler as msvc; " \
+        cmd = "import setuptools, distutils.msvc9compiler as msvc; " \
               "arch = msvc.PLAT_TO_VCVARS[msvc.get_platform()]; " \
               "env = msvc.query_vcvarsall(msvc.VERSION, arch); " \
               "print(env)"
@@ -885,7 +919,7 @@ def bash2dosPath(path):
 def do_regenerate_sysconfig():
     """
     If a Python environment has been relocated to a new folder then it's
-    possible that the sysconfig can still be usign paths for the original
+    possible that the sysconfig can still be using paths for the original
     location. Since wxPython's build uses WAF, which uses the sysconfig (via
     python-config, distutils.sysconfig, etc.) then we need to ensure that these
     paths match the current environment.
@@ -901,7 +935,7 @@ def do_regenerate_sysconfig():
         runcmd(cmd)
 
         # On success the new data module will have been written to a subfolder
-        # of the current folder, which is recorded in ./pybuilddir.tx
+        # of the current folder, which is recorded in ./pybuilddir.txt
         with open('pybuilddir.txt', 'r') as fp:
             pybd = fp.read()
 
@@ -1018,12 +1052,11 @@ def cmd_docset_py(options, args):
     # build the tarball
     msg('Archiving Phoenix docset...')
     rootname = "wxPython-docset-{}".format(cfg.VERSION)
-    tarfilename = "dist/{}.tar.gz".format(rootname)
+    tarfilename = posixjoin(phoenixDir(), 'dist', '{}.tar.gz'.format(rootname))
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
-    tarball.add(opj('dist', name+'.docset'), name+'.docset', filter=_setTarItemPerms)
-    tarball.close()
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        tarball.add(opj('dist', name+'.docset'), name+'.docset', filter=_setTarItemPerms)
 
     if options.upload:
         uploadPackage(tarfilename, options)
@@ -1144,7 +1177,7 @@ def cmd_wxlib(options, args):
     cmdTimer = CommandTimer('wxlib')
     pwd = pushDir(phoenixDir())
 
-    for wx_pkg in ['lib', 'py', 'tools']:
+    for wx_pkg in ['lib', 'py', 'svg', 'tools']:
         libDir = os.path.join(phoenixDir(), 'wx', wx_pkg)
 
         if not os.path.isdir(libDir):
@@ -1173,7 +1206,7 @@ def cmd_bdist_docs(options, args):
 
     msg("Archiving wxPython Phoenix documentation...")
     rootname = "%s-docs-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
 
     if not os.path.exists('dist'):
         os.makedirs('dist')
@@ -1252,10 +1285,14 @@ def cmd_sip(options, args):
         if not newer_group(sipFiles, sbf) and os.path.exists(pycode):
             continue
 
+        # leave this turned off for now...
+        # typehint = '-y {}'.format(posixjoin(cfg.PKGDIR, base[1:]) + '.pyi')
+        typehint = ''
+
         pycode = '-X pycode'+base+':'+pycode
         sip = getSipCmd()
-        cmd = '%s %s -c %s -b %s %s %s'  % \
-            (sip, cfg.SIPOPTS, tmpdir, sbf, pycode, src_name)
+        cmd = '%s %s -c %s -b %s %s %s %s'  % \
+            (sip, cfg.SIPOPTS, tmpdir, sbf, pycode, typehint, src_name)
         runcmd(cmd)
 
         classesNeedingClassInfo = { 'sip_corewxTreeCtrl.cpp' : 'wxTreeCtrl', }
@@ -1301,9 +1338,8 @@ def cmd_sip(options, args):
             if not os.path.exists(dest):
                 msg('%s is a new file, copying...' % os.path.basename(src))
                 srcTxt = processSrc(src, options.keep_hash_lines)
-                f = textfile_open(dest, 'wt')
-                f.write(srcTxt)
-                f.close()
+                with textfile_open(dest, 'wt') as f:
+                    f.write(srcTxt)
                 continue
 
             srcTxt = processSrc(src, options.keep_hash_lines)
@@ -1314,22 +1350,20 @@ def cmd_sip(options, args):
                 pass
             else:
                 msg('%s is changed, copying...' % os.path.basename(src))
-                f = textfile_open(dest, 'wt')
-                f.write(srcTxt)
-                f.close()
+                with textfile_open(dest, 'wt') as f:
+                    f.write(srcTxt)
 
         # Remove tmpdir and its contents
         shutil.rmtree(tmpdir)
 
 
-
 def cmd_touch(options, args):
     cmdTimer = CommandTimer('touch')
     pwd = pushDir(phoenixDir())
     etg = pathlib.Path('etg')
     for item in etg.glob('*.py'):
         item.touch()
-
+    cmd_touch_others(options, args)
 
 
 def cmd_test(options, args, tests=None):
@@ -1341,7 +1375,7 @@ def cmd_test(options, args, tests=None):
     # -n is the number of processes to run in parallel
     # --timeout will kill the test process if it gets stuck
     jobs = '-n{}'.format(options.pytest_jobs) if options.pytest_jobs else ''
-    boxed = '--boxed' if not isWindows else ''
+    boxed = '--forked' if not isWindows else ''
     sec = options.pytest_timeout
     timeout = '--timeout={}'.format(sec) if sec and sec != "0" else ''
     cmd = '"{}" -m pytest {} {} {} {} {} '.format(
@@ -1384,14 +1418,22 @@ def cmd_build_wx(options, args):
 
     if isWindows:
         # Windows-specific pre build stuff
+        if not options.no_msedge:
+            getMSWebView2()
+        else:
+            build_options.append('--no_msedge')
+
         if options.cairo:
             build_options.append('--cairo')
-            cairo_root = os.path.join(phoenixDir(), 'packaging', 'cairo-msw')
+            cairo_root = os.path.join(phoenixDir(), 'packaging', 'msw-cairo')
             os.environ['CAIRO_ROOT'] = cairo_root
 
         if options.jom:
             build_options.append('--jom')
 
+        if PY2:
+            build_options.append('--no_dpi_aware')
+
     else:
         # Platform is something other than MSW
         if options.osx_carbon:
@@ -1443,7 +1485,7 @@ def cmd_build_wx(options, args):
                 build_options.append('--gtk3')
 
         # Change to what will be the wxWidgets build folder
-        # (Note, this needs to be after any testing for file/path existance, etc.
+        # (Note, this needs to be after any testing for file/path existence, etc.
         # because they may be specified as relative paths.)
         pwd = pushDir(BUILD_DIR)
 
@@ -1515,15 +1557,19 @@ def copyWxDlls(options):
 
         # Also copy the cairo DLLs if needed
         if options.cairo:
-            cairo_root = os.path.join(phoenixDir(), 'packaging', 'cairo-msw')
+            cairo_root = os.path.join(phoenixDir(), 'packaging', 'msw-cairo')
             dlls += glob.glob(os.path.join(cairo_root, arch, 'bin', '*.dll'))
 
+        # And the webview2 (MS EDGE) DLL
+        wv2_root = os.path.join(phoenixDir(), 'packaging', 'msw-webview2')
+        dlls += glob.glob(os.path.join(wv2_root, arch, '*.dll'))
+
         # For Python 3.5 and 3.6 builds we also need to copy some VC14 redist DLLs.
         # NOTE: Do it for 3.7+ too for now. But when we fully switch over to VS 2017
         # this may need to change. See notes in wscript about it.
-        if PYVER in ['3.5', '3.6', '3.7', '3.8']:
+        if PYVER in ['3.5', '3.6', '3.7', '3.8', '3.9']:
             redist_dir = os.path.join(
-                phoenixDir(), 'packaging', 'Py3.5', 'vcredist',
+                phoenixDir(), 'packaging', 'msw-vcredist',
                 arch, 'Microsoft.VC140.CRT', '*.dll')
             dlls += glob.glob(redist_dir)
 
@@ -1568,13 +1614,15 @@ def cmd_build_py(options, args):
         os.environ['WXPYTHON_RELEASE'] = 'yes'
 
     if not isWindows:
-        WX_CONFIG = posixjoin(BUILD_DIR, 'wx-config')
-        if options.use_syswx:
-            wxcfg = posixjoin(options.prefix, 'bin', 'wx-config')
-            if options.prefix and os.path.exists(wxcfg):
-                WX_CONFIG = wxcfg
-            else:
-                WX_CONFIG = 'wx-config' # hope it is on the PATH
+        WX_CONFIG = os.environ.get('WX_CONFIG', None)
+        if WX_CONFIG is None:
+            WX_CONFIG = posixjoin(BUILD_DIR, 'wx-config')
+            if options.use_syswx:
+                wxcfg = posixjoin(options.prefix, 'bin', 'wx-config')
+                if options.prefix and os.path.exists(wxcfg):
+                    WX_CONFIG = wxcfg
+                else:
+                    WX_CONFIG = 'wx-config' # hope it is on the PATH
 
 
     wafBuildBase = wafBuildDir = getWafBuildBase()
@@ -1653,6 +1701,7 @@ def cmd_build_py(options, args):
         runcmd(cmd)
 
     copyWxDlls(options)
+    cmd_build_others(options, args)
 
     cfg = Config()
     cfg.build_locale_dir(opj(cfg.PKGDIR, 'locale'))
@@ -1665,37 +1714,25 @@ def cmd_build_py(options, args):
     print("")
 
 
-def cmd_build_vagrant(options, args):
-    # Does a build and bdist_wheel for each of the Vagrant based VMs defined
-    # in {phoenixDir}/vagrant. Requires Vagrant and VirtualBox to be installed
-    # and ready for use.  Also requires a source tarball to be present in
-    # {phoenixDir}/dist, such as what is produced with cmd_sdist.
-    cmdTimer = CommandTimer('bdist_vagrant')
+def cmd_build_docker(options, args):
+    # Uses docker images defined by the Dockerfiles under ./docker/build for
+    # building wxPython wheels for a few versions of Linux.
+    #
+    # Requirements: Docker
+    #               Internet connection for downloading docker images, if needed
+    #               One wxPython source archive in ./dist (created with cmd_sdist)
+    cmdTimer = CommandTimer('build_docker')
     cfg = Config(noWxConfig=True)
-    if not options.vagrant_vms or options.vagrant_vms == 'all':
-        VMs = [ 'centos-7     all all',
-                'debian-8     all all',
-                'debian-9     all all',
-                'fedora-26    all all',
-                'fedora-27    all gtk3', # no webkitgtk for gtk2
-                'fedora-28    all gtk3', # no webkitgtk for gtk2
-                'ubuntu-14.04 all all',
-                'ubuntu-16.04 all all',
-                'ubuntu-18.04 all all',
-                ]
-    elif options.vagrant_vms == 'none':
-        VMs = [] # to skip building anything and just upload
-    else:
-        VMs = options.vagrant_vms.split(',')
+    cmd = ['inv', 'build-wxpython']
+    if options.docker_img != 'all':
+        for img in options.docker_img.split(','):
+            cmd.append('-i')
+            cmd.append(img)
 
-    for vmName in VMs:
-        vmDir = opj(phoenixDir(), 'vagrant', vmName.split()[0])
-        pwd = pushDir(vmDir)
-        msg('Starting Vagrant VM in {}'.format(vmDir))
-        runcmd('vagrant up')
-        runcmd('vagrant ssh -c "scripts/build.sh %s"' % vmName)
-        runcmd('vagrant halt')
-        del pwd
+    # 'none' can be used to skip building and go straight to uploading
+    if options.docker_img != 'none':
+        pwd = pushDir('docker')
+        runcmd(cmd, echoCmd=True)
 
     if options.upload:
         for tag in ['gtk2', 'gtk3']:
@@ -1704,6 +1741,36 @@ def cmd_build_vagrant(options, args):
                 uploadTree(src, 'linux', options)
 
 
+def cmd_build_others(options, args):
+    # Build other stuff that may have their own separate build commands instead
+    # of the (ab)normal etg/tweak/generate/sip/compile sequence that the rest of
+    # wxPython uses. So far, it's just the wx.svg package
+    cmdTimer = CommandTimer('build_others')
+
+    cmd = [PYTHON, 'setup-wxsvg.py', 'build_ext', '--inplace']
+    if options.verbose:
+        cmd.append('--verbose')
+    runcmd(cmd)
+
+
+def cmd_touch_others(options, args):
+    cmdTimer = CommandTimer('touch_others')
+    pwd = pushDir(phoenixDir())
+    cfg = Config(noWxConfig=True)
+    pth = pathlib.Path(opj(cfg.PKGDIR, 'svg'))
+    for item in pth.glob('*.pyx'):
+        item.touch()
+
+
+def cmd_clean_others(options, args):
+    cmdTimer = CommandTimer('clean_others')
+    pwd = pushDir(phoenixDir())
+    cfg = Config(noWxConfig=True)
+    files = []
+    for wc in ['*.pyd', '*.so']:
+        files += glob.glob(opj(cfg.PKGDIR, 'svg', wc))
+    delFiles(files)
+
 
 def cmd_install(options, args):
     cmdTimer = CommandTimer('install')
@@ -1871,12 +1938,22 @@ def cmd_clean_py(options, args):
             files += glob.glob(opj(cfg.PKGDIR, wc))
     delFiles(files)
 
+    # Also remove any remaining DLLs just to make sure. This includes the C++
+    # runtime DLLs, Cairo, etc.
+    # TODO: Check for specific files, not just *.dll
+    if isWindows:
+        files = glob.glob(opj(cfg.PKGDIR, '*.dll'))
+        delFiles(files)
+
+
     if options.both:
         options.debug = False
         options.both = False
         cmd_clean_py(options, args)
         options.both = True
 
+    cmd_clean_others(options, args)
+
 
 def cmd_clean_sphinx(options, args):
     cmdTimer = CommandTimer('clean_sphinx')
@@ -1911,8 +1988,8 @@ def cmd_clean(options, args):
     cmd_clean_py(options, args)
 
 
-def cmd_clean_vagrant(options, args):
-    cmdTimer = CommandTimer('clean_vagrant')
+def cmd_clean_docker(options, args):
+    cmdTimer = CommandTimer('clean_docker')
     assert os.getcwd() == phoenixDir()
 
     d = opj(phoenixDir(), 'dist', 'linux')
@@ -1940,7 +2017,7 @@ def cmd_cleanall(options, args):
         files += glob.glob(wc)
     delFiles(files)
 
-    cmd_clean_vagrant(options, args)
+    cmd_clean_docker(options, args)
 
 
 def cmd_buildall(options, args):
@@ -1966,10 +2043,8 @@ def cmd_sdist(options, args):
         msg("Sorry, I don't know what to do in this source tree, no git workspace found.")
         return
 
-    # make a tree for building up the archive files
+    # Make a place to export everything to
     PDEST = 'build/sdist'
-    WSRC  = 'ext/wxWidgets'
-    WDEST = posixjoin(PDEST, WSRC)
     if not os.path.exists(PDEST):
         os.makedirs(PDEST)
 
@@ -1977,7 +2052,7 @@ def cmd_sdist(options, args):
     if not os.path.exists('dist'):
         os.mkdir('dist')
 
-    # recursivly export a git archive of this repo and submodules
+    # recursively export a git archive of this repo and submodules
     def _archive_submodules(root, dest):
         msg('Exporting {}...'.format(root))
         if not os.path.exists(dest):
@@ -1986,11 +2061,12 @@ def cmd_sdist(options, args):
         runcmd('git archive HEAD | tar -x -C %s' % dest, echoCmd=False)
 
         if os.path.exists('.gitmodules'):
-            for line in open('.gitmodules', 'rt').readlines():
-                line = line.strip()
-                if line.startswith('path = '):
-                    sub = line[7:]
-                    _archive_submodules(sub, opj(dest, sub))
+            with open('.gitmodules', 'rt') as fid:
+                for line in fid:
+                    line = line.strip()
+                    if line.startswith('path = '):
+                        sub = line[7:]
+                        _archive_submodules(sub, opj(dest, sub))
 
     _archive_submodules('.', os.path.abspath(PDEST))
 
@@ -2004,6 +2080,7 @@ def cmd_sdist(options, args):
         destdir = posixjoin(PDEST, cfg.PKGDIR)
         for name in glob.glob(posixjoin(cfg.PKGDIR, wc)):
             copyFile(name, destdir)
+    copyFile('demo/version.py', posixjoin(PDEST, 'demo'))
 
     # Copy the license files from wxWidgets
     msg('Copying license files...')
@@ -2038,14 +2115,14 @@ def cmd_sdist(options, args):
     # build the tarball
     msg('Archiving Phoenix source...')
     rootname = "%s-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
     pwd = pushDir(PDEST)
-    for name in glob.glob('*'):
-        tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
-    tarball.close()
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        for name in glob.glob('*'):
+            tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
+
     msg('Cleaning up...')
     del pwd
     shutil.rmtree(PDEST)
@@ -2086,18 +2163,19 @@ def cmd_sdist_demo(options, args):
 
     # Add in the README file
     copyFile('packaging/README-sdist_demo.txt', posixjoin(PDEST, 'README.txt'))
+    copyFile('demo/version.py', posixjoin(PDEST, 'demo'))
 
     # build the tarball
     msg('Archiving Phoenix demo and samples...')
     rootname = "%s-demo-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
     pwd = pushDir(PDEST)
-    for name in glob.glob('*'):
-        tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
-    tarball.close()
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        for name in glob.glob('*'):
+            tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
+
     msg('Cleaning up...')
     del pwd
     shutil.rmtree(PDEST)
@@ -2128,7 +2206,7 @@ def cmd_bdist(options, args):
     if isWindows and PYTHON_ARCH == '64bit':
         platform = 'win64'
     rootname = "%s-%s-%s-py%s" % (baseName, cfg.VERSION, platform, PYVER)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
 
     if not os.path.exists('dist'):
         os.makedirs('dist')
@@ -2136,23 +2214,23 @@ def cmd_bdist(options, args):
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
     msg("Archiving Phoenix binaries to %s..." % tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
-    tarball.add('wx', opj(rootname, 'wx'),
-                filter=lambda info: None if '.svn' in info.name \
-                                            or info.name.endswith('.pyc') \
-                                            or '__pycache__' in info.name else info)
-    tarball.add(eggInfoName, opj(rootname, eggInfoName))
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        tarball.add('wx', opj(rootname, 'wx'),
+                    filter=lambda info: None if '.svn' in info.name \
+                                                or info.name.endswith('.pyc') \
+                                                or '__pycache__' in info.name else info)
+        tarball.add(eggInfoName, opj(rootname, eggInfoName))
 
-    if not isDarwin and not isWindows and not options.no_magic and not options.use_syswx:
-        # If the DLLs are not already in the wx package folder then go fetch
-        # them now.
-        msg("Archiving wxWidgets shared libraries...")
-        dlls = glob.glob(os.path.join(wxlibdir, "*%s" % dllext))
-        for dll in dlls:
-            tarball.add(dll, os.path.join(rootname, 'wx', os.path.basename(dll)))
+        if not isDarwin and not isWindows and not options.no_magic and not options.use_syswx:
+            # If the DLLs are not already in the wx package folder then go fetch
+            # them now.
+            msg("Archiving wxWidgets shared libraries...")
+            dlls = glob.glob(os.path.join(wxlibdir, "*%s" % dllext))
+            for dll in dlls:
+                tarball.add(dll, os.path.join(rootname, 'wx', os.path.basename(dll)))
+
+        tarball.add('packaging/README-bdist.txt', os.path.join(rootname, 'README.txt'))
 
-    tarball.add('packaging/README-bdist.txt', os.path.join(rootname, 'README.txt'))
-    tarball.close()
 
     if options.upload:
         uploadPackage(tarfilename, options)
@@ -2175,10 +2253,9 @@ def cmd_setrev(options, args):
 
     else:
         svnrev = getVcsRev()
-        f = open('REV.txt', 'w')
-        svnrev = '.dev'+svnrev
-        f.write(svnrev)
-        f.close()
+        with open('REV.txt', 'w') as f:
+            svnrev = '.dev'+svnrev
+            f.write(svnrev)
         msg('REV.txt set to "%s"' % svnrev)
 
     cfg = Config()
@@ -2186,6 +2263,14 @@ def cmd_setrev(options, args):
     msg('cfg.VERSION: %s' % cfg.VERSION)
 
 
+def cmd_setpythonpath(options, args):
+    cmdTimer = CommandTimer('setpythonpath')
+    assert os.getcwd() == phoenixDir()
+
+    sys.path.insert(0, phoenixDir())
+    os.environ['PYTHONPATH'] = os.environ.get('PYTHONPATH', '') + os.pathsep + phoenixDir()
+
+
 
 #---------------------------------------------------------------------------
 
