@@ -91,8 +91,8 @@ def getWxRelease(wxRoot=None):
     if not wxRoot:
         global wxRootDir
         wxRoot = wxRootDir
-
-    configureText = open(os.path.join(wxRoot, "configure.in"), "r").read()
+    with open(os.path.join(wxRoot, "configure.in"), "r") as fid:
+        configureText = fid.read()
     majorVersion = re.search("wx_major_version_number=(\d+)", configureText).group(1)
     minorVersion = re.search("wx_minor_version_number=(\d+)", configureText).group(1)
 
@@ -226,12 +226,13 @@ def main(wxDir, args):
         "features"      : ("", "A comma-separated list of wxUSE_XYZ defines on Win, or a list of configure flags on unix."),
         "verbose"       : (False, "Print commands as they are run, (to aid with debugging this script)"),
         "jom"           : (False, "Use jom.exe instead of nmake for MSW builds."),
+        "no_dpi_aware"  : (False, "Don't use the DPI_AWARE_MANIFEST."),
+        "no_msedge"     : (False, "Do not include the MS Edge backend for wx.html2.WebView. (Windows only)"),
     }
 
     parser = optparse.OptionParser(usage="usage: %prog [options]", version="%prog 1.0")
 
-    keys = option_dict.keys()
-    for opt in sorted(keys):
+    for opt in sorted(option_dict):
         default = option_dict[opt][0]
         action = "store"
         if type(default) == bool:
@@ -292,15 +293,15 @@ def main(wxDir, args):
                             ]
 
         if sys.platform.startswith("darwin"):
-            wxpy_configure_opts.append("--disable-qtkit")
+            #wxpy_configure_opts.append("--enable-monolithic")
+            pass
         else:
             wxpy_configure_opts.append("--with-sdl")
 
-        # Set the minimum supported OSX version, and find the oldest SDK
-        # version present on the build machine.
-        # TODO: should there be a command line option to set the SDK?
+        # Set the minimum supported OSX version.
+        # TODO: Add a CLI option to set this.
         if sys.platform.startswith("darwin"):
-            wxpy_configure_opts.append("--with-macosx-version-min=10.9")
+            wxpy_configure_opts.append("--with-macosx-version-min=10.10")
             # for xcodePath in getXcodePaths():
             #     sdks = [ xcodePath+"/SDKs/MacOSX10.{}.sdk".format(n)
             #              for n in range(9, 15) ]
@@ -403,16 +404,10 @@ def main(wxDir, args):
             flags["wxUSE_DATEPICKCTRL_GENERIC"] = "1"
             flags["wxUSE_IFF"] = "1"
             flags["wxUSE_ACCESSIBILITY"] = "1"
-
-            # Remove this when Windows XP finally dies, or when there is a
-            # solution for ticket #13116...
-            flags["wxUSE_COMPILER_TLS"] = "0"
-
-            if VERSION < (2,9):
-                flags["wxUSE_DIB_FOR_BITMAP"] = "1"
-
-            if VERSION >= (2,9):
-                flags["wxUSE_UIACTIONSIMULATOR"] = "1"
+            flags["wxUSE_WINRT"] = "0"
+            flags["wxUSE_UIACTIONSIMULATOR"] = "1"
+            if not options.no_msedge:
+                flags["wxUSE_WEBVIEW_EDGE"] = "1"
 
 
         mswIncludeDir = os.path.join(wxRootDir, "include", "wx", "msw")
@@ -466,6 +461,10 @@ def main(wxDir, args):
             if options.jom:
                 nmakeCommand = 'jom.exe'
 
+            if options.no_dpi_aware:
+                args.append("USE_DPI_AWARE_MANIFEST=0")
+
+
             wxBuilder = builder.MSVCBuilder(commandName=nmakeCommand)
 
         if toolkit == "msvcProject":
@@ -586,9 +585,8 @@ def main(wxDir, args):
         for include in glob.glob(header_dir + "/*.h"):
             headers += "#include <wx/" + os.path.basename(include) + ">\n"
 
-        framework_header = open("%s.h" % fwname, "w")
-        framework_header.write(header_template % headers)
-        framework_header.close()
+        with open("%s.h" % fwname, "w") as framework_header:
+            framework_header.write(header_template % headers)
 
         run("ln -s -f %s wx" % header_dir)
         os.chdir("wx-%s/wx" % version)
