@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     23-Mar-2013
-# Copyright:   (c) 2013-2017 by Total Control Software
+# Copyright:   (c) 2013-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -23,6 +23,7 @@ ITEMS  = [ "wxTextAttrDimension",
            "wxTextAttrDimensionConverter",
            "wxTextAttrBorder",
            "wxTextAttrBorders",
+           "wxTextAttrShadow",
            "wxTextBoxAttr",
            "wxRichTextAttr",
            "wxRichTextProperties",
@@ -139,6 +140,11 @@ def run():
     c.addCppMethod('int', '__bool__', '()', "return self->IsValid();")
 
 
+    #-------------------------------------------------------
+    c = module.find('wxTextAttrShadow')
+    tools.ignoreConstOverloads(c)
+
+
     #-------------------------------------------------------
     c = module.find('wxTextBoxAttr')
     tools.ignoreConstOverloads(c)
@@ -322,6 +328,7 @@ def run():
     #-------------------------------------------------------
     c = module.find('wxRichTextImage')
     _fixDrawObject(c)
+    c.find('LoadAndScaleImageCache.changed').inOut = True
 
     #-------------------------------------------------------
     c = module.find('wxRichTextBuffer')
@@ -429,7 +436,7 @@ def run():
     #-------------------------------------------------------
     # Ignore all Dump() methods since we don't wrap wxTextOutputStream.
 
-    # TODO: try swithcing the parameter type to wxOutputStream and then in
+    # TODO: try switching the parameter type to wxOutputStream and then in
     # the wrapper code create a wxTextOutputStream from that to pass on to
     # Dump.
 
