@@ -55,8 +55,13 @@ struct wxLanguageInfo
     /// @onlyfor{wxmsw}
     wxUint32 GetLCID() const;
 
-    /// Return the locale name corresponding to this language usable with
-    /// @c setlocale() on the current system.
+    /**
+        Return the locale name corresponding to this language usable with
+        @c setlocale() on the current system.
+
+        If setting locale for this language is not supported, the returned
+        string is empty.
+     */
     wxString GetLocaleName() const;
 };
 
@@ -93,15 +98,15 @@ enum wxLocaleCategory
 
 /**
     The values understood by wxLocale::GetInfo().
-    
-    Note that for the @c wxLOCALE_*_FMT constants (the date and time formats), 
+
+    Note that for the @c wxLOCALE_*_FMT constants (the date and time formats),
     the strings returned by wxLocale::GetInfo() use strftime() or,
     equivalently, wxDateTime::Format() format. If the relevant format
     couldn't be determined, an empty string is returned -- there is no
     fallback value so that the application could determine the best course
     of actions itself in such case.
 
-    All of these values are used with @c wxLOCALE_CAT_DATE in wxLocale::GetInfo() or, 
+    All of these values are used with @c wxLOCALE_CAT_DATE in wxLocale::GetInfo() or,
     more typically, with @c wxLOCALE_CAT_DEFAULT as they only apply to a single category.
 */
 enum wxLocaleInfo
@@ -126,7 +131,7 @@ enum wxLocaleInfo
         Short date format.
 
         Notice that short and long date formats may be the same under POSIX
-        systems currently but may, and typically are, different under MSW or OS X.
+        systems currently but may, and typically are, different under MSW or macOS.
 
         @since 2.9.0
      */
@@ -427,6 +432,21 @@ public:
     static wxString GetInfo(wxLocaleInfo index,
                             wxLocaleCategory cat = wxLOCALE_CAT_DEFAULT);
 
+    /**
+        Get the values of a locale datum in the OS locale.
+
+        This function is similar to GetInfo() and, in fact, identical to it
+        under non-MSW systems. Under MSW it differs from it when no locale had
+        been explicitly set: GetInfo() returns the values corresponding to the
+        "C" locale used by the standard library functions, while this method
+        returns the values used by the OS which, in Windows case, correspond to
+        the user settings in the control panel.
+
+        @since 3.1.0
+     */
+    static wxString GetOSInfo(wxLocaleInfo index,
+                              wxLocaleCategory cat = wxLOCALE_CAT_DEFAULT);
+
     /**
         Initializes the wxLocale instance.
 
@@ -476,7 +496,7 @@ public:
 
     /**
         Check whether the operating system and/or C run time environment supports
-        this locale. For example in Windows 2000 and Windows XP, support for many
+        this locale. For example in Windows, support for many
         locales is not installed by default. Returns @true if the locale is
         supported.
 
